@@ -63,8 +63,7 @@ struct BraseroSongPropsPrivate {
 static GObjectClass *parent_class = NULL;
 
 static void
-brasero_song_props_end_changed_cb (BraseroTimeButton *button,
-				   BraseroSongProps *self)
+brasero_song_props_update_length (BraseroSongProps *self)
 {
 	gchar *length_str;
 	gint64 start;
@@ -75,32 +74,27 @@ brasero_song_props_end_changed_cb (BraseroTimeButton *button,
 	start = brasero_time_button_get_value (BRASERO_TIME_BUTTON (self->priv->start));
 	gap = gtk_spin_button_get_value (GTK_SPIN_BUTTON (self->priv->gap)) * GST_SECOND;
 
-	brasero_time_button_set_value (BRASERO_TIME_BUTTON (self->priv->start),
-				       start);
-	brasero_time_button_set_max (BRASERO_TIME_BUTTON (self->priv->start),
-				     end - 1);
-
 	length_str = brasero_utils_get_time_string (BRASERO_AUDIO_TRACK_LENGTH (start, end + gap), TRUE, FALSE);
 	gtk_label_set_markup (GTK_LABEL (self->priv->length), length_str);
 	g_free (length_str);
 }
 
+static void
+brasero_song_props_end_changed_cb (BraseroTimeButton *button,
+				   BraseroSongProps *self)
+{
+	gint64 end;
+
+	end = brasero_time_button_get_value (BRASERO_TIME_BUTTON (self->priv->end));
+	brasero_time_button_set_max (BRASERO_TIME_BUTTON (self->priv->start), end - 1);
+	brasero_song_props_update_length (self);
+}
+
 static void
 brasero_song_props_start_changed_cb (BraseroTimeButton *button,
 				     BraseroSongProps *self)
 {
-	gchar *length_str;
-	gint64 start;
-	gint64 end;
-	gint64 gap;
-
-	end = brasero_time_button_get_value (BRASERO_TIME_BUTTON (self->priv->end));
-	start = brasero_time_button_get_value (BRASERO_TIME_BUTTON (self->priv->start));
-	gap = gtk_spin_button_get_value (GTK_SPIN_BUTTON (self->priv->gap)) * GST_SECOND;
-
-	length_str = brasero_utils_get_time_string (BRASERO_AUDIO_TRACK_LENGTH (start, end + gap), TRUE, FALSE);
-	gtk_label_set_markup (GTK_LABEL (self->priv->length), length_str);
-	g_free (length_str);
+	brasero_song_props_update_length (self);
 }
 
 static void
@@ -181,12 +175,14 @@ brasero_song_props_init (BraseroSongProps *obj)
 	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 3, 4, GTK_FILL, GTK_FILL, 0, 0);
 	gtk_table_attach_defaults (GTK_TABLE (table), obj->priv->isrc, 1, 2, 3, 4);
 
+	gtk_widget_show_all (frame);
+
 	/* second part of the dialog */
 	table = gtk_table_new (2, 4, FALSE);
 	gtk_table_set_row_spacings (GTK_TABLE (table), 6);
 	gtk_table_set_col_spacings (GTK_TABLE (table), 6);
 
-	frame = brasero_utils_pack_properties (_("<big><b>Options</b></big>"),
+	frame = brasero_utils_pack_properties (_("<b>Options</b>"),
 					       table,
 					       NULL);
 	gtk_container_set_border_width (GTK_CONTAINER (frame), 6);
@@ -221,6 +217,8 @@ brasero_song_props_init (BraseroSongProps *obj)
 	gtk_misc_set_alignment (GTK_MISC (obj->priv->length), 0.0, 0.5);
 	gtk_table_attach (GTK_TABLE (table), obj->priv->length, 1, 2, 3, 4, GTK_FILL, GTK_FILL, 0, 0);
 
+	gtk_widget_show_all (frame);
+
 	/* monitor since there must be 4 sec at least for a track */
 	g_signal_connect (obj->priv->end,
 			  "value-changed",
@@ -290,7 +288,7 @@ brasero_song_props_set_properties (BraseroSongProps *self,
 	gchar *string;
 	gdouble secs;
 
-	string = g_strdup_printf (_("<b><big>Song information for track %02i</big></b>"), track_num);
+	string = g_strdup_printf (_("<b>Song information for track %02i</b>"), track_num);
 	gtk_label_set_markup (GTK_LABEL (self->priv->label), string);
 	g_free (string);
 
@@ -311,8 +309,17 @@ brasero_song_props_set_properties (BraseroSongProps *self,
 
 	brasero_time_button_set_max (BRASERO_TIME_BUTTON (self->priv->start), end - 1);
 	brasero_time_button_set_value (BRASERO_TIME_BUTTON (self->priv->start), start);
+
+	g_signal_handlers_block_by_func (self->priv->end,
+					 brasero_song_props_end_changed_cb,
+					 self);
 	brasero_time_button_set_max (BRASERO_TIME_BUTTON (self->priv->end), length);
 	brasero_time_button_set_value (BRASERO_TIME_BUTTON (self->priv->end), end);
+	g_signal_handlers_unblock_by_func (self->priv->end,
+					   brasero_song_props_end_changed_cb,
+					   self);
+
+	brasero_song_props_update_length (self);
 }
 
 static void
