@@ -29,11 +29,14 @@
 #include <string.h>
 #include <errno.h>
 #include <fcntl.h>
+#include <unistd.h>
 
 #include <glib.h>
 #include <glib-object.h>
 #include <glib/gi18n-lib.h>
 
+#include <gio/gio.h>
+
 #include "burn-basics.h"
 #include "burn-debug.h"
 #include "burn-session.h"
@@ -43,7 +46,7 @@
 #include "burn-task-ctx.h"
 #include "burn-task-item.h"
 #include "brasero-marshal.h"
-#include "brasero-ncb.h"
+#include "burn-drive.h"
 #include "burn-medium.h"
 
 typedef struct _BraseroJobOutput {
@@ -79,10 +82,10 @@ struct BraseroJobPrivate {
 	BraseroJob *linked;
 };
 
-#define BRASERO_JOB_DEBUG(job)	brasero_job_log_message (job, G_STRLOC,	\
-				"%s called %s", 			\
-				G_OBJECT_TYPE_NAME (job),		\
-				G_STRFUNC);
+#define BRASERO_JOB_DEBUG(job_MACRO)	brasero_job_log_message (job_MACRO, G_STRLOC,	\
+					"%s called %s", 			\
+					G_OBJECT_TYPE_NAME (job_MACRO),		\
+					G_STRFUNC);
 
 #define BRASERO_JOB_PRIVATE(o)  (G_TYPE_INSTANCE_GET_PRIVATE ((o), BRASERO_TYPE_JOB, BraseroJobPrivate))
 
@@ -359,11 +362,12 @@ brasero_job_check_output_disc_space (BraseroJob *self,
 				     GError **error)
 {
 	BraseroBurnSession *session;
-	NautilusBurnDrive *drive;
 	gint64 output_blocks = 0;
 	gint64 media_blocks = 0;
 	BraseroJobPrivate *priv;
 	BraseroBurnFlag flags;
+	BraseroMedium *medium;
+	BraseroDrive *drive;
 
 	priv = BRASERO_JOB_PRIVATE (self);
 
@@ -373,17 +377,17 @@ brasero_job_check_output_disc_space (BraseroJob *self,
 
 	session = brasero_task_ctx_get_session (priv->ctx);
 	drive = brasero_burn_session_get_burner (session);
+	medium = brasero_drive_get_medium (drive);
 	flags = brasero_burn_session_get_flags (session);
 
 	/* FIXME: if we can't recover the size of the medium 
-	 * what should we do ? do as if we could ?
-	 */
+	 * what should we do ? do as if we could ? */
 
 	/* see if we are appending or not */
 	if (flags & (BRASERO_BURN_FLAG_APPEND|BRASERO_BURN_FLAG_MERGE))
-		NCB_MEDIA_GET_FREE_SPACE (drive, NULL, &media_blocks);
+		brasero_medium_get_free_space (medium, NULL, &media_blocks);
 	else
-		NCB_MEDIA_GET_CAPACITY (drive, NULL, &media_blocks);
+		brasero_medium_get_capacity (medium, NULL, &media_blocks);
 
 	/* this is not really an error, we'll probably ask the 
 	 * user to load a new disc */
@@ -407,13 +411,12 @@ static BraseroBurnResult
 brasero_job_check_output_volume_space (BraseroJob *self,
 				       GError **error)
 {
-	gchar *uri_str;
+	GFile *file;
+	GFileInfo *info;
 	gchar *directory;
-	GnomeVFSURI *uri;
+	guint64 vol_size = 0;
 	gint64 output_size = 0;
 	BraseroJobPrivate *priv;
-	GnomeVFSFileSize vol_size = 0;
-	BraseroBurnResult result = BRASERO_BURN_ERR;
 
 	/* now that the job has a known output we must check that the volume the
 	 * job is writing to has enough space for all output */
@@ -425,21 +428,23 @@ brasero_job_check_output_volume_space (BraseroJob *self,
 		return BRASERO_BURN_ERR;
 
 	directory = g_path_get_dirname (priv->output->image);
-
-	uri_str = gnome_vfs_get_uri_from_local_path (directory);
+	file = g_file_new_for_path (directory);
 	g_free (directory);
 
-	uri = gnome_vfs_uri_new (uri_str);
-	g_free (uri_str);
-
-	if (uri == NULL)
+	if (file == NULL)
 		goto error;
 
-	result = gnome_vfs_get_volume_free_space (uri, &vol_size);
-	if (result != GNOME_VFS_OK)
+	info = g_file_query_filesystem_info (file,
+					     G_FILE_ATTRIBUTE_FILESYSTEM_FREE,
+					     NULL,
+					     error);
+	if (!info)
 		goto error;
 
-	gnome_vfs_uri_unref (uri);
+	g_object_unref (file);
+
+	vol_size = g_file_info_get_attribute_uint64 (info, G_FILE_ATTRIBUTE_FILESYSTEM_FREE);
+	g_object_unref (info);
 
 	/* get the size of the output this job is supposed to create */
 	brasero_job_get_session_output_size (BRASERO_JOB (self), NULL, &output_size);
@@ -460,12 +465,12 @@ brasero_job_check_output_volume_space (BraseroJob *self,
 
 error:
 
-	g_set_error (error,
-		     BRASERO_BURN_ERROR,
-		     BRASERO_BURN_ERROR_GENERAL,
-		     _("the size of the volume can't be checked (%s)"),
-		     gnome_vfs_result_to_string (result));
-	gnome_vfs_uri_unref (uri);
+	if (error && *error == NULL)
+		g_set_error (error,
+			     BRASERO_BURN_ERROR,
+			     BRASERO_BURN_ERROR_GENERAL,
+			     _("the size of the volume can't be checked (Unknown error)"));
+	g_object_unref (file);
 	return BRASERO_BURN_ERR;
 }
 
@@ -528,6 +533,7 @@ brasero_job_set_output_file (BraseroJob *self,
 	priv->output->toc = toc;
 
 	flags = brasero_burn_session_get_flags (session);
+
 	if (flags & BRASERO_BURN_FLAG_CHECK_SIZE)
 		return brasero_job_check_output_volume_space (self, error);
 
@@ -764,8 +770,10 @@ brasero_job_item_stop (BraseroTaskItem *item,
 
 	brasero_job_disconnect (self, error);
 
-	g_object_unref (priv->ctx);
-	priv->ctx = NULL;
+	if (priv->ctx) {
+		g_object_unref (priv->ctx);
+		priv->ctx = NULL;
+	}
 
 	return BRASERO_BURN_OK;
 }
@@ -783,6 +791,47 @@ brasero_job_iface_init_task_item (BraseroTaskItemIFace *iface)
 	iface->clock_tick = brasero_job_item_clock_tick;
 }
 
+BraseroBurnResult
+brasero_job_tag_lookup (BraseroJob *self,
+			const gchar *tag,
+			GValue **value)
+{
+	BraseroJobPrivate *priv;
+	BraseroBurnSession *session;
+
+	BRASERO_JOB_DEBUG (self);
+
+	priv = BRASERO_JOB_PRIVATE (self);
+
+	session = brasero_task_ctx_get_session (priv->ctx);
+	return brasero_burn_session_tag_lookup (session,
+						tag,
+						value);
+}
+
+BraseroBurnResult
+brasero_job_tag_add (BraseroJob *self,
+		     const gchar *tag,
+		     GValue *value)
+{
+	BraseroJobPrivate *priv;
+	BraseroBurnSession *session;
+
+	BRASERO_JOB_DEBUG (self);
+
+	priv = BRASERO_JOB_PRIVATE (self);
+
+	if (!brasero_job_is_last_active (self))
+		return BRASERO_BURN_ERR;
+
+	session = brasero_task_ctx_get_session (priv->ctx);
+	brasero_burn_session_tag_add (session,
+				      tag,
+				      value);
+
+	return BRASERO_BURN_OK;
+}
+
 /**
  * Means a job successfully completed its task.
  * track can be NULL, depending on whether or not the job created a track.
@@ -1277,12 +1326,32 @@ brasero_job_get_action (BraseroJob *self, BraseroJobAction *action)
 	return BRASERO_BURN_OK;
 }
 
+BraseroBurnResult
+brasero_job_get_bus_target_lun (BraseroJob *self, gchar **BTL)
+{
+	BraseroBurnSession *session;
+	BraseroJobPrivate *priv;
+	BraseroDrive *drive;
+
+	BRASERO_JOB_DEBUG (self);
+
+	g_return_val_if_fail (BTL != NULL, BRASERO_BURN_ERR);
+
+	priv = BRASERO_JOB_PRIVATE (self);
+	session = brasero_task_ctx_get_session (priv->ctx);
+
+	drive = brasero_burn_session_get_burner (session);
+	*BTL = brasero_drive_get_bus_target_lun_string (drive);;
+
+	return BRASERO_BURN_OK;
+}
+
 BraseroBurnResult
 brasero_job_get_device (BraseroJob *self, gchar **device)
 {
 	BraseroBurnSession *session;
-	NautilusBurnDrive *drive;
 	BraseroJobPrivate *priv;
+	BraseroDrive *drive;
 	const gchar *path;
 
 	BRASERO_JOB_DEBUG (self);
@@ -1293,7 +1362,7 @@ brasero_job_get_device (BraseroJob *self, gchar **device)
 	session = brasero_task_ctx_get_session (priv->ctx);
 
 	drive = brasero_burn_session_get_burner (session);
-	path = NCB_DRIVE_GET_DEVICE (drive);
+	path = brasero_drive_get_device (drive);
 	*device = g_strdup (path);
 
 	return BRASERO_BURN_OK;
@@ -1320,8 +1389,9 @@ BraseroBurnResult
 brasero_job_get_last_session_address (BraseroJob *self, gint64 *address)
 {
 	BraseroBurnSession *session;
-	NautilusBurnDrive *drive;
 	BraseroJobPrivate *priv;
+	BraseroMedium *medium;
+	BraseroDrive *drive;
 
 	BRASERO_JOB_DEBUG (self);
 
@@ -1330,7 +1400,8 @@ brasero_job_get_last_session_address (BraseroJob *self, gint64 *address)
 	priv = BRASERO_JOB_PRIVATE (self);
 	session = brasero_task_ctx_get_session (priv->ctx);
 	drive = brasero_burn_session_get_burner (session);
-	NCB_MEDIA_GET_LAST_DATA_TRACK_ADDRESS (drive, NULL, address);
+	medium = brasero_drive_get_medium (drive);
+	brasero_medium_get_last_data_track_address (medium, NULL, address);
 
 	return BRASERO_BURN_OK;
 }
@@ -1339,8 +1410,9 @@ BraseroBurnResult
 brasero_job_get_next_writable_address (BraseroJob *self, gint64 *address)
 {
 	BraseroBurnSession *session;
-	NautilusBurnDrive *drive;
 	BraseroJobPrivate *priv;
+	BraseroMedium *medium;
+	BraseroDrive *drive;
 
 	BRASERO_JOB_DEBUG (self);
 
@@ -1349,7 +1421,8 @@ brasero_job_get_next_writable_address (BraseroJob *self, gint64 *address)
 	priv = BRASERO_JOB_PRIVATE (self);
 	session = brasero_task_ctx_get_session (priv->ctx);
 	drive = brasero_burn_session_get_burner (session);
-	*address = NCB_MEDIA_GET_NEXT_WRITABLE_ADDRESS (drive);
+	medium = brasero_drive_get_medium (drive);
+	*address = brasero_medium_get_next_writable_address (medium);
 
 	return BRASERO_BURN_OK;
 }
@@ -1386,7 +1459,7 @@ brasero_job_get_speed (BraseroJob *self, guint *speed)
 	rate = brasero_burn_session_get_rate (session);
 
 	media = brasero_burn_session_get_dest_media (session);
-	if (media & BRASERO_MEDIUM_DVD)
+	if (media & (BRASERO_MEDIUM_DVD|BRASERO_MEDIUM_DVD_DL))
 		*speed = BRASERO_RATE_TO_SPEED_DVD (rate);
 	else 
 		*speed = BRASERO_RATE_TO_SPEED_CD (rate);
@@ -1398,8 +1471,9 @@ BraseroBurnResult
 brasero_job_get_max_rate (BraseroJob *self, guint64 *rate)
 {
 	BraseroBurnSession *session;
-	NautilusBurnDrive *drive;
 	BraseroJobPrivate *priv;
+	BraseroMedium *medium;
+	BraseroDrive *drive;
 
 	BRASERO_JOB_DEBUG (self);
 
@@ -1409,7 +1483,9 @@ brasero_job_get_max_rate (BraseroJob *self, guint64 *rate)
 	session = brasero_task_ctx_get_session (priv->ctx);
 
 	drive = brasero_burn_session_get_burner (session);
-	*rate = NCB_MEDIA_GET_MAX_WRITE_RATE  (drive);
+	medium = brasero_drive_get_medium (drive);
+
+	*rate = brasero_medium_get_max_write_speed (medium);
 
 	return BRASERO_BURN_OK;
 }
@@ -1418,8 +1494,9 @@ BraseroBurnResult
 brasero_job_get_max_speed (BraseroJob *self, guint *speed)
 {
 	BraseroBurnSession *session;
-	NautilusBurnDrive *drive;
 	BraseroJobPrivate *priv;
+	BraseroMedium *medium;
+	BraseroDrive *drive;
 	BraseroMedia media;
 	guint64 rate;
 
@@ -1431,12 +1508,12 @@ brasero_job_get_max_speed (BraseroJob *self, guint *speed)
 	session = brasero_task_ctx_get_session (priv->ctx);
 
 	drive = brasero_burn_session_get_burner (session);
-	rate = NCB_MEDIA_GET_MAX_WRITE_RATE  (drive);
-	media = NCB_MEDIA_GET_STATUS (drive);
-	if (media & BRASERO_MEDIUM_DVD)
-		*speed = NAUTILUS_BURN_DRIVE_DVD_SPEED (rate);
+	rate = brasero_medium_get_max_write_speed (medium);
+	media = brasero_medium_get_status (medium);
+	if (media & (BRASERO_MEDIUM_DVD|BRASERO_MEDIUM_DVD_DL))
+		*speed = BRASERO_RATE_TO_SPEED_DVD (rate);
 	else 
-		*speed = NAUTILUS_BURN_DRIVE_CD_SPEED (rate);
+		*speed = BRASERO_RATE_TO_SPEED_CD (rate);
 
 	return BRASERO_BURN_OK;
 }
@@ -1577,7 +1654,7 @@ brasero_job_set_progress (BraseroJob *self,
 	if (priv->next)
 		return BRASERO_BURN_ERR;
 
-	if (progress < 0.0)
+	if (progress < 0.0 || progress > 1.0)
 		return BRASERO_BURN_ERR;
 
 	return brasero_task_ctx_set_progress (priv->ctx, progress);
