@@ -228,14 +228,28 @@ brasero_genisoimage_set_argv_image (BraseroGenisoimage *genisoimage,
 		BRASERO_JOB_NOT_READY (genisoimage);
 
 	brasero_track_get_type (track, &type);
-	if ((type.subtype.fs_type & BRASERO_IMAGE_FS_JOLIET))
+	if (type.subtype.fs_type & BRASERO_IMAGE_FS_JOLIET)
 		g_ptr_array_add (argv, g_strdup ("-J"));
 
-	if ((type.subtype.fs_type & BRASERO_IMAGE_FS_VIDEO))
+	if ((type.subtype.fs_type & BRASERO_IMAGE_FS_ISO)
+	&&  (type.subtype.fs_type & BRASERO_IMAGE_ISO_FS_LEVEL_3)) {
+		g_ptr_array_add (argv, g_strdup ("-iso-level"));
+		g_ptr_array_add (argv, g_strdup ("3"));
+
+		/* NOTE the following is specific to genisoimage */
+		/* g_ptr_array_add (argv, g_strdup ("-allow-limited-size")); */
+	}
+
+	if (type.subtype.fs_type & BRASERO_IMAGE_FS_UDF)
+		g_ptr_array_add (argv, g_strdup ("-udf"));
+
+	if (type.subtype.fs_type & BRASERO_IMAGE_FS_VIDEO)
 		g_ptr_array_add (argv, g_strdup ("-dvd-video"));
 
 	g_ptr_array_add (argv, g_strdup ("-graft-points"));
-	g_ptr_array_add (argv, g_strdup ("-D"));	// This is dangerous the manual says but apparently it works well
+
+	if (type.subtype.fs_type & BRASERO_IMAGE_ISO_FS_DEEP_DIRECTORY)
+		g_ptr_array_add (argv, g_strdup ("-D"));	// This is dangerous the manual says but apparently it works well
 
 	result = brasero_job_get_tmp_file (BRASERO_JOB (genisoimage),
 					   NULL,
@@ -281,19 +295,6 @@ brasero_genisoimage_set_argv_image (BraseroGenisoimage *genisoimage,
 	g_ptr_array_add (argv, g_strdup ("-exclude-list"));
 	g_ptr_array_add (argv, excluded_path);
 
-	brasero_job_get_action (BRASERO_JOB (genisoimage), &action);
-	if (action == BRASERO_JOB_ACTION_SIZE) {
-		g_ptr_array_add (argv, g_strdup ("-quiet"));
-		g_ptr_array_add (argv, g_strdup ("-print-size"));
-
-		brasero_job_set_current_action (BRASERO_JOB (genisoimage),
-						BRASERO_BURN_ACTION_GETTING_SIZE,
-						NULL,
-						FALSE);
-		brasero_job_start_progress (BRASERO_JOB (genisoimage), FALSE);
-		return BRASERO_BURN_OK;
-	}
-
 	brasero_job_get_data_label (BRASERO_JOB (genisoimage), &label);
 	if (label) {
 		g_ptr_array_add (argv, g_strdup ("-V"));
@@ -347,6 +348,19 @@ brasero_genisoimage_set_argv_image (BraseroGenisoimage *genisoimage,
 		}
 	}
 
+	brasero_job_get_action (BRASERO_JOB (genisoimage), &action);
+	if (action == BRASERO_JOB_ACTION_SIZE) {
+		g_ptr_array_add (argv, g_strdup ("-quiet"));
+		g_ptr_array_add (argv, g_strdup ("-print-size"));
+
+		brasero_job_set_current_action (BRASERO_JOB (genisoimage),
+						BRASERO_BURN_ACTION_GETTING_SIZE,
+						NULL,
+						FALSE);
+		brasero_job_start_progress (BRASERO_JOB (genisoimage), FALSE);
+		return BRASERO_BURN_OK;
+	}
+
 	if (brasero_job_get_fd_out (BRASERO_JOB (genisoimage), NULL) != BRASERO_BURN_OK) {
 		gchar *output = NULL;
 
@@ -479,7 +493,9 @@ brasero_genisoimage_export_caps (BraseroPlugin *plugin, gchar **error)
 				  BRASERO_MEDIUM_CDRW|
 				  BRASERO_MEDIUM_DVDR|
 				  BRASERO_MEDIUM_DVDRW|
+				  BRASERO_MEDIUM_DVD_DL|
 				  BRASERO_MEDIUM_DVDR_PLUS|
+				  BRASERO_MEDIUM_RESTRICTED|
 				  BRASERO_MEDIUM_APPENDABLE|
 				  BRASERO_MEDIUM_HAS_AUDIO|
 				  BRASERO_MEDIUM_HAS_DATA,
@@ -488,6 +504,8 @@ brasero_genisoimage_export_caps (BraseroPlugin *plugin, gchar **error)
 				  BRASERO_BURN_FLAG_NONE);
 
 	input = brasero_caps_data_new (BRASERO_IMAGE_FS_ISO|
+				       BRASERO_IMAGE_FS_UDF|
+				       BRASERO_IMAGE_ISO_FS_LEVEL_3|
 				       BRASERO_IMAGE_FS_JOLIET|
 				       BRASERO_IMAGE_FS_VIDEO);
 
